@@ -277,15 +277,14 @@ void SystemImpl__toWindowsSeperators(char* buffer, int bufferLength)
 int SystemImpl__chdir(const char* path)
 {
 #if defined(__MINGW32__) || defined(_MSC_VER)
-  MULTIBYTE_TO_WIDECHAR_LENGTH(path, unicodePathLength);
-  MULTIBYTE_TO_WIDECHAR_VAR(path, unicodePath, unicodePathLength);
+  wchar_t* unicodePath = omc_multibyte_to_wchar_str(path);
+  int success = SetCurrentDirectoryW(unicodePath);
+  free(unicodePath);
 
-  if (!SetCurrentDirectoryW(unicodePath)) {
-    MULTIBYTE_OR_WIDECHAR_VAR_FREE(unicodePath);
+  if (!success) {
     c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("SetCurrentDirectoryW failed."),NULL,0);
     return -1;
   }
-  MULTIBYTE_OR_WIDECHAR_VAR_FREE(unicodePath);
   return 0;
 #else
   if (chdir(path) != 0) {
@@ -306,16 +305,20 @@ extern char* SystemImpl__pwd(void)
     return NULL;
   }
 
-  WCHAR unicodePath[bufLen];
+  wchar_t* unicodePath = (wchar_t*)omc_alloc_interface.malloc_atomic(bufLen * sizeof(wchar_t));
   if (!GetCurrentDirectoryW(bufLen, unicodePath)) {
     c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("GetCurrentDirectoryW failed."),NULL,0);
     return NULL;
   }
-  WIDECHAR_TO_MULTIBYTE_LENGTH(unicodePath, bufferLength);
-  WIDECHAR_TO_MULTIBYTE_VAR(unicodePath, buffer, bufferLength);
+
+  char* buffer = omc_wchar_to_multibyte_str(unicodePath);
+  // This seems like it is an overkill. If all we need is the length then strlen on the 'buffer' above should suffice.
+  // I am leaving it like this for now since this was what was being done by the macro WIDECHAR_TO_MULTIBYTE_LENGTH
+  int bufferLength = WideCharToMultiByte(CP_UTF8, 0, unicodePath, -1, NULL, 0, NULL, NULL);
   SystemImpl__toWindowsSeperators(buffer, bufferLength);
   char *res = omc_alloc_interface.malloc_strdup(buffer);
-  MULTIBYTE_OR_WIDECHAR_VAR_FREE(buffer);
+  free(buffer);
+  GC_free(unicodePath);
   return res;
 #else
   char buf[MAXPATHLEN];
@@ -333,12 +336,9 @@ extern int SystemImpl__regularFileExists(const char* str)
   WIN32_FIND_DATAW FileData;
   HANDLE sh;
 
-  MULTIBYTE_TO_WIDECHAR_LENGTH(str, unicodeFilenameLength);
-  MULTIBYTE_TO_WIDECHAR_VAR(str, unicodeFilename, unicodeFilenameLength);
-
+  wchar_t* unicodeFilename = omc_multibyte_to_wchar_str(str);
   sh = FindFirstFileW(unicodeFilename, &FileData);
-
-  MULTIBYTE_OR_WIDECHAR_VAR_FREE(unicodeFilename);
+  free(unicodeFilename);
 
   if (sh == INVALID_HANDLE_VALUE) {
     if (strlen(str) >= MAXPATHLEN)
@@ -356,9 +356,9 @@ extern int SystemImpl__regularFileExists(const char* str)
   FindClose(sh);
   return ((FileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0);
 #else
-  struct stat buf;
+  omc_stat_t buf;
   /* adrpo: TODO: check if str leads to a path > PATH_MAX, maybe use realpath impl. from below */
-  if (stat(str, &buf)) return 0;
+  if (omc_stat(str, &buf)) return 0;
   return (buf.st_mode & S_IFREG) != 0;
 #endif
 }
@@ -380,11 +380,7 @@ static char* SystemImpl__readFile(const char* filename)
   char* buf;
   int res;
   FILE * file = NULL;
-#if defined(__MINGW32__) || defined(_MSC_VER)
-  struct _stat statstr;
-#else
-  struct stat statstr;
-#endif
+  omc_stat_t statstr;
   res = omc_stat(filename, &statstr);
 
   if (res != 0) {
@@ -425,7 +421,7 @@ static char* SystemImpl__readFile(const char* filename)
   }
   buf = (char*) omc_alloc_interface.malloc_atomic(statstr.st_size+1);
 
-  if( (res = fread(buf, sizeof(char), statstr.st_size, file)) != statstr.st_size) {
+  if( (res = omc_fread(buf, sizeof(char), statstr.st_size, file, 0)) != statstr.st_size) {
     const char *c_tokens[2]={strerror(errno),filename};
     c_add_message(NULL,85, /* ERROR_OPENING_FILE */
       ErrorType_scripting,
@@ -503,7 +499,7 @@ int SystemImpl__writeFile(const char* filename, const char* data)
 int SystemImpl__appendFile(const char* filename, const char *data)
 {
   FILE *file = NULL;
-  file = fopen(filename, "a");
+  file = omc_fopen(filename, "a");
 
   if(file == NULL) {
     const char *c_tokens[1] = {filename};
@@ -601,13 +597,10 @@ int runProcess(const char* cmd)
   si.cb = sizeof(si);
   ZeroMemory(&pi, sizeof(pi));
 
-
   sprintf(command, "%s \"%s\"", c, cmd);
-
   /* fprintf(stderr, "%s\n", command); fflush(NULL); */
 
-  MULTIBYTE_TO_WIDECHAR_LENGTH(command, unicodeCommandLength);
-  MULTIBYTE_TO_WIDECHAR_VAR(command, unicodeCommand, unicodeCommandLength);
+  wchar_t* unicodeCommand = omc_multibyte_to_wchar_str(command);
 
   if (CreateProcessW(NULL, unicodeCommand, NULL, NULL, FALSE, CREATE_NO_WINDOW, NULL, NULL, &si, &pi))
   {
@@ -617,7 +610,8 @@ int runProcess(const char* cmd)
     CloseHandle(pi.hProcess);
     CloseHandle(pi.hThread);
   }
-  MULTIBYTE_OR_WIDECHAR_VAR_FREE(unicodeCommand);
+
+  free(unicodeCommand);
   GC_free(command);
   return (int)exitCode;
 }
@@ -874,11 +868,7 @@ int SystemImpl__spawnCall(const char* path, const char* str)
 
 int SystemImpl__plotCallBackDefined(threadData_t *threadData)
 {
-  if (threadData->plotClassPointer && threadData->plotCB) {
-    return 1;
-  } else {
-    return 0;
-  }
+  return threadData->plotClassPointer && threadData->plotCB;
 }
 
 void SystemImpl__plotCallBack(threadData_t *threadData, int externalWindow, const char* filename, const char* title, const char* grid, const char* plotType,
@@ -886,13 +876,26 @@ void SystemImpl__plotCallBack(threadData_t *threadData, int externalWindow, cons
                               const char* y2, const char* curveWidth, const char* curveStyle, const char* legendPosition, const char* footer, const char* autoScale,
                               const char* variables)
 {
-  if (threadData->plotClassPointer && threadData->plotCB) {
+  if (SystemImpl__plotCallBackDefined(threadData)) {
     PlotCallback pcb = threadData->plotCB;
     pcb(threadData->plotClassPointer, externalWindow, filename, title, grid, plotType, logX, logY, xLabel, yLabel, x1, x2, y1, y2, curveWidth, curveStyle,
         legendPosition, footer, autoScale, variables);
   }
 }
 
+int SystemImpl__loadModelCallBackDefined(threadData_t *threadData)
+{
+  return threadData->loadModelClassPointer && threadData->loadModelCB;
+}
+
+void SystemImpl__loadModelCallBack(threadData_t *threadData, const char* modelname)
+{
+  if (SystemImpl__loadModelCallBackDefined(threadData)) {
+    LoadModelCallback cb = threadData->loadModelCB;
+    cb(threadData->loadModelClassPointer, modelname);
+  }
+}
+
 extern double SystemImpl__time(void)
 {
   clock_t cl = clock();
@@ -904,21 +907,29 @@ extern int SystemImpl__directoryExists(const char *str)
   /* if the string is NULL return 0 */
   if (!str) return 0;
 #if defined(__MINGW32__) || defined(_MSC_VER)
-  WIN32_FIND_DATA FileData;
+  WIN32_FIND_DATAW FileData;
   HANDLE sh;
   char* path = strdup(str);
   int last = strlen(path)-1;
   /* adrpo: RTFM! the path cannot end in a slash??!! https://msdn.microsoft.com/en-us/library/windows/desktop/aa364418(v=vs.85).aspx */
-  if (last > 0 && (path[last] == '\\' || path[last] == '/')) path[last] = '\0';
-  sh = FindFirstFile(path, &FileData);
+  while (last > 0 && (path[last] == '\\' || path[last] == '/'))
+      last--;
+
+  path[last + 1] = '\0';
+
+  wchar_t* unicodePath = omc_multibyte_to_wchar_str(path);
+  sh = FindFirstFileW(unicodePath, &FileData);
+  free(unicodePath);
   free(path);
+
   if (sh == INVALID_HANDLE_VALUE)
     return 0;
+
   FindClose(sh);
   return (FileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0;
 #else
-  struct stat buf;
-  if (stat(str, &buf))
+  omc_stat_t buf;
+  if (omc_stat(str, &buf))
     return 0;
   return (buf.st_mode & S_IFDIR) != 0;
 #endif
@@ -950,7 +961,7 @@ extern int SystemImpl__copyFile(const char *str_1, const char *str_2)
   char buf[8192];
   FILE *source, *target;
 
-  source = fopen(str_1, "r");
+  source = omc_fopen(str_1, "r");
   if (source==0) {
     const char *msg[2] = {strerror(errno), str_1};
     c_add_message(NULL,85,
@@ -961,7 +972,7 @@ extern int SystemImpl__copyFile(const char *str_1, const char *str_2)
       2);
     return 0;
   }
-  target = fopen(str_2, "w");
+  target = omc_fopen(str_2, "w");
   if (target==0) {
     const char *msg[2] = {strerror(errno), str_2};
     c_add_message(NULL,85,
@@ -974,7 +985,7 @@ extern int SystemImpl__copyFile(const char *str_1, const char *str_2)
     return 0;
   }
 
-  while (( n = fread(buf, 1, 8192, source) )) {
+  while (( n = omc_fread(buf, 1, 8192, source, 1) )) {
     if (n != fwrite(buf, 1, n, target)) {
       rv = 0;
       break;
@@ -1043,10 +1054,10 @@ static int SystemImpl__removeDirectoryItem(const char *path)
       buf = (char *)omc_alloc_interface.malloc_atomic(len);
       if (buf != NULL)
       {
-        struct stat statbuf;
+        omc_stat_t statbuf;
 
         snprintf(buf, len, "%s/%s", path, p->d_name);
-        if (stat(buf, &statbuf) == 0)
+        if (omc_stat(buf, &statbuf) == 0)
         {
           if (S_ISDIR(statbuf.st_mode))
           {
@@ -1054,7 +1065,7 @@ static int SystemImpl__removeDirectoryItem(const char *path)
           }
           else
           {
-            r2 = unlink(buf);
+            r2 = omc_unlink(buf);
           }
         }
       }
@@ -1071,7 +1082,7 @@ static int SystemImpl__removeDirectoryItem(const char *path)
   else
   {
     /* Could not open path as dir, try to handle as file */
-    retval = unlink(path);
+    retval = omc_unlink(path);
   }
 
   return retval;
@@ -1176,13 +1187,13 @@ extern int SystemImpl__removeDirectory(const char *path)
           if (strcmp(p->d_name+(len-len_post), pat_post) == 0)
           {
             /* pre and post pattern do match */
-            struct stat statbuf;
+            omc_stat_t statbuf;
             char * newdir = (char *)omc_alloc_interface.malloc_atomic(len_base+len+len_sub+3);
 
             strcpy(newdir, basepath);
             strcat(newdir, "/");
             strcat(newdir, p->d_name);
-            if (stat(newdir, &statbuf) == 0)
+            if (omc_stat(newdir, &statbuf) == 0)
             {
               if (S_ISDIR(statbuf.st_mode))
               {
@@ -1197,7 +1208,7 @@ extern int SystemImpl__removeDirectory(const char *path)
               {
                 if (sub == NULL)
                 {
-                  unlink(newdir);
+                  omc_unlink(newdir);
                 }
                 else
                 {
@@ -1225,11 +1236,7 @@ extern const char* SystemImpl__readFileNoNumeric(const char* filename)
   char* buf, *bufRes;
   int res,numCount;
   FILE * file = NULL;
-#if defined(__MINGW32__) || defined(_MSC_VER)
-  struct _stat statstr;
-#else
-  struct stat statstr;
-#endif
+  omc_stat_t statstr;
   res = omc_stat(filename, &statstr);
 
   if(res!=0) {
@@ -1246,7 +1253,7 @@ extern const char* SystemImpl__readFileNoNumeric(const char* filename)
   file = omc_fopen(filename,"rb");
   buf = (char*) omc_alloc_interface.malloc_atomic(statstr.st_size+1);
   bufRes = (char*) omc_alloc_interface.malloc_atomic((statstr.st_size+70)*sizeof(char));
-  if( (res = fread(buf, sizeof(char), statstr.st_size, file)) != statstr.st_size) {
+  if( (res = omc_fread(buf, sizeof(char), statstr.st_size, file, 0)) != statstr.st_size) {
     fclose(file);
     return "Failed while reading file";
   }
@@ -1496,13 +1503,13 @@ int file_select_directories(direntry entry)
 {
   char fileName[MAXPATHLEN];
   int res;
-  struct stat fileStatus;
+  omc_stat_t fileStatus;
   if ((strcmp(entry->d_name, ".") == 0) ||
       (strcmp(entry->d_name, "..") == 0)) {
     return (0);
   } else {
     sprintf(fileName,"%s/%s",select_from_dir,entry->d_name);
-    res = stat(fileName,&fileStatus);
+    res = omc_stat(fileName,&fileStatus);
     if (res!=0) return 0;
     if ((fileStatus.st_mode & _IFDIR))
       return (1);
@@ -2117,7 +2124,12 @@ int splitVersion(const char *version, long *versionNum, char **versionExtra)
     buf = next;
   } while (cont && ++i < MODELICAPATH_LEVELS);
   if (*buf == ' ') buf++;
-  *versionExtra = omc_alloc_interface.malloc_strdup(buf);
+
+  if (*buf == '+') {
+    *versionExtra = omc_alloc_interface.malloc_strdup("");
+  } else {
+    *versionExtra = omc_alloc_interface.malloc_strdup(buf);
+  }
   len = strlen(*versionExtra);
   /* fprintf(stderr, "have len %ld versionExtra %s\n", len, *versionExtra); */
   if (len >= 2 && 0==strcmp("mo", *versionExtra+len-2)) {
@@ -2869,8 +2881,8 @@ int SystemImpl__fileIsNewerThan(const char *file1, const char *file2)
   FindClose(sh2);
   return ((LARGE_INTEGER*)&ftWrite1)->QuadPart - ((LARGE_INTEGER*)&ftWrite2)->QuadPart > 0 ? 1 : 0;
 #else
-  struct stat buf1, buf2;
-  if (stat(file1, &buf1)) {
+  omc_stat_t buf1, buf2;
+  if (omc_stat(file1, &buf1)) {
     const char *c_tokens[2]={strerror(errno),file1};
     c_add_message(NULL,85,
         ErrorType_scripting,
@@ -2880,7 +2892,7 @@ int SystemImpl__fileIsNewerThan(const char *file1, const char *file2)
         2);
     return -1;
   }
-  if (stat(file2, &buf2)) {
+  if (omc_stat(file2, &buf2)) {
     const char *c_tokens[2]={strerror(errno),file2};
     c_add_message(NULL,85,
         ErrorType_scripting,
@@ -2913,25 +2925,24 @@ int SystemImpl__fileContentsEqual(const char *file1, const char *file2)
   char buf1[8192],buf2[8192];
   FILE *f1,*f2;
   int i1,i2,totalread=0,error=0;
-#if !defined(_MSC_VER)
-  struct stat stbuf1;
-  struct stat stbuf2;
-  if (stat(file1, &stbuf1)) return 0;
-  if (stat(file2, &stbuf2)) return 0;
-  if (stbuf1.st_size != stbuf2.st_size) return 0;
-#endif
-  f1 = fopen(file1,"rb");
+  omc_stat_t stbuf1;
+  omc_stat_t stbuf2;
+
+  if (omc_stat(file1, &stbuf1) || omc_stat(file2, &stbuf2) || stbuf1.st_size != stbuf2.st_size)
+    return 0;
+
+  f1 = omc_fopen(file1,"rb");
   if (f1 == NULL) {
     return 0;
   }
-  f2 = fopen(file2,"rb");
+  f2 = omc_fopen(file2,"rb");
   if (f2 == NULL) {
     fclose(f1);
     return 0;
   }
   do {
-    i1 = fread(buf1,1,8192,f1);
-    i2 = fread(buf2,1,8192,f2);
+    i1 = omc_fread(buf1,1,8192,f1, 1);
+    i2 = omc_fread(buf2,1,8192,f2, 1);
     if (i1 != i2 || strncmp(buf1,buf2,i1)) {
       error = 1;
     }
@@ -2989,16 +3000,34 @@ static char *omc_strtok_r(char *str, const char *delim, char **saveptr)
 
 #endif /* defined(__MINGW32__) */
 
-int SystemImpl__stat(const char *filename, double *size, double *mtime)
+typedef enum {
+    FileType_NoFile = 1,
+    FileType_RegularFile,
+    FileType_Directory,
+    FileType_SpecialFile
+} OpenModelicaFileType;
+
+int SystemImpl__stat(const char *filename, double *size, double *mtime, int *fileType)
 {
-  struct stat stats;
-  if (0 != stat(filename, &stats)) {
+  omc_stat_t stats;
+  if (0 != omc_stat(filename, &stats)) {
     *size = 0;
     *mtime = 0;
+    *fileType = FileType_NoFile;
     return 0;
   }
   *size = stats.st_size;
   *mtime = stats.st_mtime;
+  if (S_ISREG(stats.st_mode)) {
+    *fileType = FileType_RegularFile;
+  }
+  else if (S_ISDIR(stats.st_mode)) {
+    *fileType = FileType_Directory;
+  }
+  else {
+    *fileType = FileType_SpecialFile;
+  }
+
   return 1;
 }
 
@@ -3043,15 +3072,15 @@ int SystemImpl__covertTextFileToCLiteral(const char *textFile, const char *outFi
   int result = 0, n, i, j, k, isMSVC = !strcmp(target, "msvc");
   char buffer[512];
   char obuffer[1024];
-  fin = fopen(textFile, "r");
+  fin = omc_fopen(textFile, "r");
   if (!fin) {
     goto done;
   }
   errno = 0;
 #if defined(__APPLE_CC__)||defined(__MINGW32__)||defined(__MINGW64__)
-  unlink(outFile);
+  omc_unlink(outFile);
 #endif
-  fout = fopen(outFile, "w");
+  fout = omc_fopen(outFile, "w");
   if (!fout) {
     const char *c_token[1]={strerror(errno)};
     c_add_message(NULL,85,
@@ -3068,7 +3097,7 @@ int SystemImpl__covertTextFileToCLiteral(const char *textFile, const char *outFi
     fputc('{', fout);
     fputc('\n', fout);
     do {
-      n = fread(buffer,1,511,fin);
+      n = omc_fread(buffer,1,511,fin, 1);
       j = 0;
       /* adrpo: encode each char */
       for (i=0; i<n; i++) {
@@ -3111,7 +3140,7 @@ int SystemImpl__covertTextFileToCLiteral(const char *textFile, const char *outFi
   {
     fputc('\"', fout);
     do {
-      n = fread(buffer,1,511,fin);
+      n = omc_fread(buffer,1,511,fin, 1);
       j = 0;
       for (i=0; i<n; i++) {
         switch (buffer[i]) {
